# robotic_complex/gripper_node.py

import rclpy
from rclpy.node import Node
from std_msgs.msg import Int32
from pymodbus.client import ModbusTcpClient
from pymodbus.exceptions import ModbusException


class GripperNode(Node):
    def __init__(self):
        super().__init__('gripper_node')

        # Параметры подключения
        self.declare_parameter('ip', '10.27.1.80')
        self.declare_parameter('port', 502)

        self.ip = self.get_parameter('ip').value
        self.port = self.get_parameter('port').value

        # Подключение к захвату
        self.client = ModbusTcpClient(self.ip, self.port)
        if not self.client.connect():
            self.get_logger().error("Не удалось подключиться к захвату.")
            self.destroy_node()
            return
        else:
            self.get_logger().info(f"Подключен к захвату по адресу {self.ip}:{self.port}")

        # Инициализация захвата
        self.initialize_gripper()

        # Подписка на топики
        self.subscription_position = self.create_subscription(
            Int32,
            'gripper/position',
            self.position_callback,
            10)

        self.subscription_force = self.create_subscription(
            Int32,
            'gripper/force',
            self.force_callback,
            10)

    def initialize_gripper(self):
        try:
            value = 0xA5  # Полная инициализация
            address = 0x0100
            self.client.write_register(address, value, unit=1)
            self.get_logger().info("Захват инициализирован.")
        except ModbusException as e:
            self.get_logger().error(f"Ошибка инициализации захвата: {e}")

    def position_callback(self, msg):
        position = msg.data
        if not 0 <= position <= 1000:
            self.get_logger().error('Позиция должна быть в диапазоне 0-1000.')
            return
        try:
            address = 0x0103
            self.client.write_register(address, position, unit=1)
            self.get_logger().info(f"Позиция захвата установлена на {position}.")
        except ModbusException as e:
            self.get_logger().error(f"Ошибка установки позиции: {e}")

    def force_callback(self, msg):
        force = msg.data
        if not 20 <= force <= 100:
            self.get_logger().error('Сила должна быть в диапазоне 20-100.')
            return
        try:
            address = 0x0101
            self.client.write_register(address, force, unit=1)
            self.get_logger().info(f"Сила захвата установлена на {force}%.")
        except ModbusException as e:
            self.get_logger().error(f"Ошибка установки силы: {e}")

    def destroy_node(self):
        self.client.close()
        super().destroy_node()


def main(args=None):
    rclpy.init(args=args)
    node = GripperNode()
    if node:
        rclpy.spin(node)
        node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()