# robotic_complex/manipulator_node.py

import rclpy
from rclpy.node import Node
from std_msgs.msg import Float64MultiArray
import socket
import time


class ManipulatorNode(Node):
    def __init__(self):
        super().__init__('manipulator_node')

        # Параметры подключения
        self.declare_parameter('ip', '192.168.1.201')
        self.declare_parameter('port', 6000)

        self.ip = self.get_parameter('ip').value
        self.port = self.get_parameter('port').value

        # Подключение к манипулятору
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.sock.connect((self.ip, self.port))
            self.get_logger().info(f"Подключен к манипулятору по адресу {self.ip}:{self.port}")
        except Exception as e:
            self.get_logger().error(f"Ошибка подключения к манипулятору: {e}")
            self.destroy_node()
            return

        # Подписка на топик с углами
        self.subscription = self.create_subscription(
            Float64MultiArray,
            'manipulator/angles',
            self.listener_callback,
            10)
        self.subscription  # предотвращаем предупреждение об отсутствии использования переменной

    def listener_callback(self, msg):
        if len(msg.data) != 6:
            self.get_logger().error('Ожидается массив из 6 углов.')
            return

        for angle in msg.data:
            padded, _ = self.pad_number(angle)
            message = padded.encode('utf-8')
            try:
                self.sock.sendall(message)
                time.sleep(0.05)
            except Exception as e:
                self.get_logger().error(f"Ошибка отправки данных: {e}")

    def pad_number(self, number):
        integer_part, fractional_part = self.get_integer_and_fractional_parts(number)
        pad_length = 6
        integer_part = str(integer_part)[:pad_length].rjust(pad_length, '0')
        fractional_part = str(fractional_part)[:pad_length].ljust(pad_length, '0')
        prefix = "+" if number >= 0 else "-"
        return (f"{prefix}{integer_part}.{fractional_part}", 2 + pad_length * 2)

    def get_integer_and_fractional_parts(self, number):
        integer_fractional = str(abs(number)).split('.')
        if len(integer_fractional) == 1:
            integer_fractional.append('0')
        return list(map(int, integer_fractional))

    def destroy_node(self):
        self.sock.close()
        super().destroy_node()


def main(args=None):
    rclpy.init(args=args)
    node = ManipulatorNode()
    if node:
        rclpy.spin(node)
        node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()